@@ -22,17 +22,21 @@
 
 #ifdef ENABLE_CHILLIQUERY
 void garden_print_list(int fd, pass_through *ptlist, int ptcnt) {
+  char mask[32];
   char line[512];
   pass_through *pt;
   int i;
 
   for (i = 0; i < ptcnt; i++) {
     pt = &ptlist[i];
-
+    
+    safe_strncpy(mask, inet_ntoa(pt->mask), sizeof(mask));
+    
     safe_snprintf(line, sizeof(line),
-		  "host=%-16s proto=%-3d port=%-3d\n", inet_ntoa(pt->host),
+		  "host=%-16s mask=%-16s proto=%-3d port=%-3d\n", 
+		  inet_ntoa(pt->host), mask,
 		  pt->proto, pt->port);
-
+    
     if (!write(fd, line, strlen(line))) /* error */;
   }
 }
@@ -123,7 +127,7 @@ int pass_throughs_from_string(pass_through *ptlist, uint32_t ptlen,
     memset(&pt, 0, sizeof(pass_through));
     
     /* eat whitespace */
-    while (isspace(*p1)) p1++;
+    while (isspace((int) *p1)) p1++;
     
     /* look for specific protocols */
     if ((t = strchr(p1, ':'))) { 
@@ -260,7 +264,7 @@ void garden_load_domainfile() {
     
     while ((read = getline(&line, &len, fp)) != -1) {
       if (read <= 0) continue;
-      else if (!line[0] || line[0] == '#' || isspace(line[0])) continue;
+      else if (!line[0] || line[0] == '#' || isspace((int) line[0])) continue;
       else {
 	
 	uamdomain_regex * uam_re = (uamdomain_regex *)
@@ -268,7 +272,7 @@ void garden_load_domainfile() {
 
 	char * pline = line;
 	
-	while (isspace(pline[read-1]))
+	while (isspace((int) pline[read-1]))
 	  pline[--read] = 0;
 
 	if (pline[0] == '!') {
