@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,15 +16,9 @@
  * 
  */
 
-
 #ifndef _GARDEN_H_
 #define _GARDEN_H_
 
-#ifdef ENABLE_CHILLIREDIR
-#include <sys/types.h>
-#include <regex.h>
-#endif
-
 typedef struct pass_through_t {
   struct in_addr host;              /* IP or Network */
   struct in_addr mask;              /* Netmask */
@@ -40,10 +34,11 @@ typedef struct regex_pass_through_t {
   regex_t re_host;
   regex_t re_path;
   regex_t re_qs;
+  char inuse:1;
   char neg_host:1;
   char neg_path:1;
   char neg_qs:1;
-  char reserved:5;
+  char reserved:4;
 } regex_pass_through;
 
 int regex_pass_throughs_from_string(regex_pass_through *ptlist, uint32_t ptlen, uint32_t *ptcnt, char *s);
@@ -52,4 +47,16 @@ int regex_pass_throughs_from_string(regex_pass_through *ptlist, uint32_t ptlen,
 int pass_through_add(pass_through *ptlist, uint32_t ptlen, uint32_t *ptcnt, pass_through *pt);
 int pass_throughs_from_string(pass_through *ptlist, uint32_t ptlen, uint32_t *ptcnt, char *s);
 
+int garden_check(pass_through *ptlist, int ptcnt, uint8_t *pack, int dst);
+
+#ifdef ENABLE_CHILLIQUERY
+void garden_print(int fd);
+#endif
+
+#ifdef ENABLE_UAMDOMAINFILE
+void garden_load_domainfile();
+void garden_free_domainfile();
+int  garden_check_domainfile(char *question);
+#endif
+
 #endif
