@@ -31,7 +31,7 @@ static const char *description =
   "  http://www.coova.org/\n";
 
 static const char *copyright = 
-  "Copyright (c) 2003-2005 Mondru AB., 2006-2011 Coova Technologies LLC.\n"
+  "2006-2011 Coova Technologies LLC., Copyright (c) 2003-2005 Mondru AB.\n"
   "Licensed under the GNU General Public License (GPL).\n";
 
 static const char *usage =
@@ -47,6 +47,12 @@ static const char *compile_options = "Compiled with "
 #ifdef ENABLE_CHILLIPROXY
   "ENABLE_CHILLIPROXY "
 #endif
+#ifdef ENABLE_COA
+  "ENABLE_COA "
+#endif
+#ifdef ENABLE_RADPROXY
+  "ENABLE_RADPROXY "
+#endif
 #ifdef ENABLE_CHILLIRADSEC
   "ENABLE_CHILLIRADSEC "
 #endif
@@ -125,6 +131,12 @@ static const char *compile_options = "Compiled with "
 #ifdef ENABLE_STATFILE
   "ENABLE_STATFILE "
 #endif
+#ifdef ENABLE_UAMANYIP
+  "ENABLE_UAMANYIP "
+#endif
+#ifdef ENABLE_UAMUIPORT
+  "ENABLE_UAMUIPORT "
+#endif
 #ifdef HAVE_MATRIXSSL
   "HAVE_MATRIXSSL "
 #endif
@@ -138,7 +150,7 @@ static const char *compile_options = "Compiled with "
 
 char *STRDUP(char *s) {
   if (!s) return 0;
-  while (isspace(*s)) s++;
+  while (isspace((int) *s)) s++;
   if (!*s) return 0;
   return s;
 }
@@ -280,7 +292,9 @@ int main(int argc, char **argv) {
   _options.macreauth = args_info.macreauth_flag;
   _options.macauthdeny = args_info.macauthdeny_flag;
   _options.uamport = args_info.uamport_arg;
+#ifdef ENABLE_UAMUIPORT
   _options.uamuiport = args_info.uamuiport_arg;
+#endif
   _options.macallowlocal = args_info.macallowlocal_flag;
   _options.strictmacauth = args_info.strictmacauth_flag;
   _options.strictdhcp = args_info.strictdhcp_flag;
@@ -425,7 +439,7 @@ int main(int argc, char **argv) {
 
     /* Replace anything but hex with space */
     for (i=0; i<macstrlen; i++) 
-      if (!isxdigit(macstr[i])) 
+      if (!isxdigit((int) macstr[i])) 
 	macstr[i] = 0x20;
 
     if (sscanf (macstr, "%2x %2x %2x %2x %2x %2x", 
@@ -462,7 +476,8 @@ int main(int argc, char **argv) {
 
     /* Replace anything but hex with space */
     for (i=0; i<macstrlen; i++) 
-      if (!isxdigit(macstr[i])) macstr[i] = 0x20;
+      if (!isxdigit((int) macstr[i])) 
+	macstr[i] = 0x20;
 
     if (sscanf (macstr, "%2x %2x %2x %2x %2x %2x", 
 		&temp[0], &temp[1], &temp[2], 
@@ -585,8 +600,10 @@ int main(int argc, char **argv) {
   }
 
   _options.uamanydns = args_info.uamanydns_flag;
+#ifdef ENABLE_UAMANYIP
   _options.uamanyip = args_info.uamanyip_flag;
   _options.uamnatanyip = args_info.uamnatanyip_flag;
+#endif
   _options.dnsparanoia = args_info.dnsparanoia_flag;
   _options.radiusoriginalurl = args_info.radiusoriginalurl_flag;
   _options.routeonetone = args_info.routeonetone_flag;
@@ -692,9 +709,11 @@ int main(int argc, char **argv) {
 
   _options.allowdyn = 1;
   
+#ifdef ENABLE_UAMANYIP
   _options.autostatip = args_info.autostatip_arg;
   if (_options.autostatip)
     _options.uamanyip = 1;
+#endif
   
   if (args_info.nodynip_flag) {
     _options.allowdyn = 0;
@@ -727,15 +746,24 @@ int main(int argc, char **argv) {
     _options.allowstat = 0;
   }
 
-  /* anyipexclude */
-  if (args_info.anyipexclude_arg) {
-    if (option_aton(&_options.anyipexclude_addr, 
-		    &_options.anyipexclude_mask, 
-		    args_info.anyipexclude_arg, 0)) {
-      log_err(0, "Failed to parse anyip exclude network!");
+#ifdef ENABLE_UAMANYIP
+  if (args_info.uamnatanyipex_arg) {
+    if (option_aton(&_options.uamnatanyipex_addr, 
+		    &_options.uamnatanyipex_mask, 
+		    args_info.uamnatanyipex_arg, 0)) {
+      log_err(0, "Failed to parse uamnatanyipex network!");
       return -1;
     }
   }
+  if (args_info.uamanyipex_arg) {
+    if (option_aton(&_options.uamanyipex_addr, 
+		    &_options.uamanyipex_mask, 
+		    args_info.uamanyipex_arg, 0)) {
+      log_err(0, "Failed to parse uamanyipex network!");
+      return -1;
+    }
+  }
+#endif
   
   if (args_info.dns1_arg) {
     if (!inet_aton(args_info.dns1_arg, &_options.dns1)) {
@@ -922,7 +950,7 @@ int main(int argc, char **argv) {
       else {
 	/* Replace anything but hex and comma with space */
 	for (i=0; i<strlen(p1); i++) 
-	  if (!isxdigit(p1[i])) p1[i] = 0x20;
+	  if (!isxdigit((int) p1[i])) p1[i] = 0x20;
       
 	if (sscanf (p1, "%2x %2x %2x %2x %2x %2x",
 		    &mac[0], &mac[1], &mac[2], &mac[3], &mac[4], &mac[5]) != 6) {
@@ -1047,6 +1075,9 @@ int main(int argc, char **argv) {
   _options.uamhostname = STRDUP(args_info.uamhostname_arg);
   _options.binconfig = STRDUP(args_info.bin_arg);
   _options.ethers = STRDUP(args_info.ethers_arg);
+#ifdef ENABLE_REDIRINJECT
+  _options.inject = STRDUP(args_info.inject_arg);
+#endif
 
   ret = 0;
 
