@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2009-2010 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2009-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,11 +14,13 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
+ * Author: David Bird <david@coova.com>
  */
 
 #define MAIN_FILE
 
 #include "chilli.h"
+#include "debug.h"
 
 #ifdef USING_CURL
 #include <curl/curl.h>
@@ -79,11 +81,27 @@ static CURLM * curl_multi;
 static int still_running = 0;
 #endif
 
+static void print_requests() {
+  proxy_request * req = 0;
+  int i;
+
+  for (i=0; i < max_requests; i++) {
+    req = &requests[i];
+    log_info("%.3d. inuse=%d prev=%.3d next=%.3d url=%s fd=%d", 
+	     req->index, req->inuse,
+	     req->prev ? req->prev->index : -1,
+	     req->next ? req->next->index : -1,
+	     req->url ? (char *)req->url->data : "",
+	     req->conn.sock);
+  }
+}
+
 static proxy_request * get_request() {
   proxy_request * req = 0;
   int i;
 
-  if (!max_requests) {
+  if (!requests) {
+
     /* Initialize linked list */
     max_requests = 16; /* hard maximum! (should be configurable) */
 
@@ -108,14 +126,17 @@ static proxy_request * get_request() {
     num_requests_free--;
   }
   
+#if(_debug_)
+  log_dbg("connections free %d", num_requests_free);
+#endif
+  
   if (!req) {
     /* problem */
     log_err(0,"out of connections");
+    print_requests();
     return 0;
   }
 
-  log_dbg("connections free %d", num_requests_free);
-  
   req->next = req->prev = 0;
   req->inuse = 1;
   return req;
@@ -158,7 +179,9 @@ static void close_request(proxy_request *req) {
   requests_free = req;
   num_requests_free++;
 
+#if(_debug_)
   log_dbg("connections free %d", num_requests_free);
+#endif
 }
 
 static int http_aaa_finish(proxy_request *req) {
@@ -166,6 +189,9 @@ static int http_aaa_finish(proxy_request *req) {
   struct radius_t *radius = req->radius;
 
 #ifdef USING_CURL
+#if(_debug_)
+  log_dbg("calling curl_easy_cleanup()");
+#endif
   curl_multi_remove_handle(curl_multi, req->curl);
   curl_easy_cleanup(req->curl);
   req->curl = 0;
@@ -174,19 +200,25 @@ static int http_aaa_finish(proxy_request *req) {
 #endif
 
   if (req->data->slen) {
+#if(_debug_)
     log_dbg("Received: %s\n",req->data->data);
+#endif
     req->authorized = !memcmp(req->data->data, "Auth: 1", 7);
   }
 
   /* initialize response packet */
   switch(req->radius_req.code) {
   case RADIUS_CODE_ACCOUNTING_REQUEST:
+#if(_debug_)
     log_dbg("Accounting-Response");
+#endif
     radius_default_pack(radius, &req->radius_res, RADIUS_CODE_ACCOUNTING_RESPONSE);
     break;
     
   case RADIUS_CODE_ACCESS_REQUEST:
+#if(_debug_)
     log_dbg("Access-%s", req->authorized ? "Accept" : "Reject");
+#endif
     radius_default_pack(radius, &req->radius_res, 
 			req->authorized ? RADIUS_CODE_ACCESS_ACCEPT :
 			RADIUS_CODE_ACCESS_REJECT);
@@ -269,7 +301,9 @@ static int http_aaa_finish(proxy_request *req) {
 		uint32_t v = (uint32_t) atoi(ptr+strlen(attrs[i].n));
 		if (v > 0) {
 		  radius_addattr(radius, &req->radius_res, attrs[i].a, attrs[i].v, attrs[i].va, v, NULL, 0);
+#if(_debug_)
 		  log_dbg("Setting %s = %d", attrs[i].n, v);
+#endif
 		}
 	      }
 	      break;
@@ -277,7 +311,9 @@ static int http_aaa_finish(proxy_request *req) {
 	      {
 		radius_addattr(radius, &req->radius_res, attrs[i].a, attrs[i].v, attrs[i].va, 0, 
 			       (uint8_t *)ptr+strlen(attrs[i].n), strlen(ptr)-strlen(attrs[i].n));
+#if(_debug_)
 		log_dbg("Setting %s = %s", attrs[i].n, ptr+strlen(attrs[i].n));
+#endif
 	      }
 	      break;
 	    }
@@ -340,10 +376,12 @@ static int http_aaa_setup(struct radius_t *radius, proxy_request *req) {
 
   char *user = 0;
   char *pwd = 0;
+#ifdef HAVE_OPENSSL
   char *ca = 0;
   char *cert = 0;
   char *key = 0;
   char *keypwd = 0;
+#endif
 
   req->radius = radius;
 
@@ -362,17 +400,23 @@ static int http_aaa_setup(struct radius_t *radius, proxy_request *req) {
 #ifdef HAVE_OPENSSL
 
     if (cert && strlen(cert)) {
+#if(_debug_)
       log_dbg("using cert [%s]",cert);
+#endif
       curl_easy_setopt(curl, CURLOPT_SSLCERT, cert);
       curl_easy_setopt(curl, CURLOPT_SSLCERTTYPE, "PEM");
     }
 
     if (key && strlen(key)) {
+#if(_debug_)
       log_dbg("using key [%s]",key);
+#endif
       curl_easy_setopt(curl, CURLOPT_SSLKEY, key);
       curl_easy_setopt(curl, CURLOPT_SSLKEYTYPE, "PEM");
       if (keypwd && strlen(keypwd)) {
+#if(_debug_)
 	log_dbg("using key pwd [%s]",keypwd);
+#endif
 #ifdef CURLOPT_SSLCERTPASSWD
 	curl_easy_setopt(curl, CURLOPT_SSLCERTPASSWD, keypwd);
 #else
@@ -389,7 +433,9 @@ static int http_aaa_setup(struct radius_t *radius, proxy_request *req) {
 
     if (ca && strlen(ca)) {
 #ifdef CURLOPT_ISSUERCERT
+#if(_debug_)
       log_dbg("using ca [%s]",ca);
+#endif
       curl_easy_setopt(curl, CURLOPT_ISSUERCERT, ca);
       curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 1);
 #else
@@ -464,13 +510,13 @@ static int http_aaa_setup(struct radius_t *radius, proxy_request *req) {
   req->radius = radius;
   req->past_headers = 0;
   req->nline = 0;
-
+  
   if (get_urlparts((char *) req->url->data, hostname, sizeof(hostname), &port, &uripos))
     return -1;
-
+  
   if (!req->wbuf) 
     req->wbuf = bfromcstr("");
-
+  
   bassignformat(req->wbuf, 
 		"GET %s HTTP/1.1\r\n"
 		"Host: %s:%d\r\n"
@@ -478,28 +524,32 @@ static int http_aaa_setup(struct radius_t *radius, proxy_request *req) {
 		"Connection: close\r\n"
 		"\r\n",
 		req->url->data + uripos, hostname, port);
-
+  
   if (conn_setup(&req->conn, hostname, port, req->wbuf))
     return -1;
-
+  
   conn_set_readhandler(&req->conn, http_conn_read, req);
   conn_set_donehandler(&req->conn, http_conn_finish, req);
-
+  
   return 0;
 }
 #endif
 
 static int http_aaa(struct radius_t *radius, proxy_request *req) {
-
+  
   if (http_aaa_setup(radius, req) == 0) {
     
 #ifdef USING_CURL
     curl_multi_add_handle(curl_multi, req->curl);
-
+    
     while(CURLM_CALL_MULTI_PERFORM ==
 	  curl_multi_perform(curl_multi, &still_running));
-#endif
 
+#if(_debug_ > 1)
+    log_dbg("curl still running %d", still_running);
+#endif
+#endif
+    
     return 0;
   }
 
@@ -543,13 +593,13 @@ static void http_aaa_register(int argc, char **argv, int i) {
 
     int fd = socket(AF_INET, SOCK_DGRAM, 0);
 
-    strncpy(ifr.ifr_name, _options.dhcpif, sizeof(ifr.ifr_name));
+    safe_strncpy(ifr.ifr_name, _options.dhcpif, sizeof(ifr.ifr_name));
 
     if (ioctl(fd, SIOCGIFHWADDR, (caddr_t)&ifr) == 0) {
       memcpy(nas_hwaddr, ifr.ifr_hwaddr.sa_data, PKT_ETH_ALEN);
-      sprintf(mac, "%.2X-%.2X-%.2X-%.2X-%.2X-%.2X", 
-	      nas_hwaddr[0],nas_hwaddr[1],nas_hwaddr[2],
-	      nas_hwaddr[3],nas_hwaddr[4],nas_hwaddr[5]);
+      safe_snprintf(mac, sizeof(mac), "%.2X-%.2X-%.2X-%.2X-%.2X-%.2X", 
+		    nas_hwaddr[0],nas_hwaddr[1],nas_hwaddr[2],
+		    nas_hwaddr[3],nas_hwaddr[4],nas_hwaddr[5]);
       bcatcstr(req.url, mac);
     }
       
@@ -585,13 +635,17 @@ static void http_aaa_register(int argc, char **argv, int i) {
 
   if (http_aaa_setup(0, &req) == 0) {
 
+#if(_debug_ > 1)
     log_dbg("==> %s\npost:%s", req.url->data, req.post->data);
+#endif
 
 #ifdef USING_CURL
     curl_easy_perform(req.curl);
 #endif
 
+#if(_debug_ > 1)
     log_dbg("<== %s", req.data->data);
+#endif
 
 #ifdef USING_CURL
     curl_easy_cleanup(req.curl);
@@ -895,8 +949,11 @@ static void process_radius(struct radius_t *radius, struct radius_packet_t *pack
     if (_options.uamsecret && _options.uamsecret[0])
       redir_md_param(req->url, _options.uamsecret, "&");
     
+#if(_debug_ > 1)
     log_dbg("==> %s", req->url->data);
-    http_aaa(radius, req);
+#endif
+    if (http_aaa(radius, req) < 0)
+      close_request(req);
     
   } else {
     log_err(0, "problem: %s", error);
@@ -937,6 +994,7 @@ int main(int argc, char **argv) {
   selfpipe = selfpipe_init();
 
   chilli_signals(&keep_going, &reload_config);
+  selfpipe_trap(SIGUSR2);
 
   /*
    *  Support a --register mode whereby all subsequent arguments are 
@@ -959,14 +1017,14 @@ int main(int argc, char **argv) {
 
   if (radius_new(&radius_auth, &radiuslisten, 
 		 _options.radiusauthport ? _options.radiusauthport : RADIUS_AUTHPORT, 
-		 0, NULL, 0, NULL, NULL, NULL)) {
+		 0, 0)) {
     log_err(0, "Failed to create radius");
     return -1;
   }
 
   if (radius_new(&radius_acct, &radiuslisten, 
 		 _options.radiusacctport ? _options.radiusacctport : RADIUS_ACCTPORT, 
-		 0, NULL, 0, NULL, NULL, NULL)) {
+		 0, 0)) {
     log_err(0, "Failed to create radius");
     return -1;
   }
@@ -1015,7 +1073,7 @@ int main(int argc, char **argv) {
     
     timeout.tv_sec = 1;
     timeout.tv_usec = 0;
- 
+
     status = select(maxfd + 1, &fdread, &fdwrite, &fdexcep, &timeout);
 
     switch (status) {
@@ -1033,7 +1091,16 @@ int main(int argc, char **argv) {
 	socklen_t fromlen = sizeof(addr);
 	
 	if (FD_ISSET(selfpipe, &fdread)) {
-	  chilli_handle_signal(0, 0);
+	  int signo = chilli_handle_signal(0, 0);
+	  if (signo) {
+#if(_debug_)
+	    log_dbg("main-proxy signal %d", signo);
+#endif
+	    switch(signo) {
+	    case SIGUSR2: print_requests(); break;
+	    default: break;
+	    }
+	  }
 	}
 
 	if (FD_ISSET(radius_auth->fd, &fdread)) {
@@ -1056,7 +1123,9 @@ int main(int argc, char **argv) {
 	   *    ---> Accounting
 	   */
 	  
+#if(_debug_)
 	  log_dbg("received accounting");
+#endif
 	  
 	  if ((status = recvfrom(radius_acct->fd, &radius_pack, sizeof(radius_pack), 0, 
 			       (struct sockaddr *) &addr, &fromlen)) <= 0) {
@@ -1069,12 +1138,19 @@ int main(int argc, char **argv) {
       }
       
 #ifdef USING_CURL
-      if (still_running) {
-	while(CURLM_CALL_MULTI_PERFORM ==
-	      curl_multi_perform(curl_multi, &still_running));
-      }
+      while(CURLM_CALL_MULTI_PERFORM ==
+	    curl_multi_perform(curl_multi, &still_running));
 
+#if(_debug_ > 1)
+      log_dbg("curl still running %d", still_running);
+#endif
+      
       while ((msg = curl_multi_info_read(curl_multi, &msgs_left))) {
+
+#if(_debug_ > 1)
+	log_dbg("curl messages left %d", msgs_left);
+#endif
+
 	if (msg->msg == CURLMSG_DONE) {
 	  
 	  int found = 0;
@@ -1085,8 +1161,12 @@ int main(int argc, char **argv) {
 	  
 	  if (found) {
 	    --idx;
+#if(_debug_)
 	    log_dbg("HTTP completed with status %d\n", msg->data.result);
+#endif
 	    http_aaa_finish(&requests[idx]);
+	  } else {
+	    log_err(0, "Could not find request in queue");
 	  }
 	}
       }
@@ -1100,6 +1180,9 @@ int main(int argc, char **argv) {
     }
   }
 
+  radius_free(radius_auth);
+  radius_free(radius_acct);
+
 #ifdef USING_CURL
   curl_multi_cleanup(curl_multi);
   curl_global_cleanup();
