@@ -166,9 +166,7 @@ int options_load(int argc, char **argv, bstring bt) {
       if (done_before) break;
       else {
 	char *offline = getenv("CHILLI_OFFLINE");
-
 	if (offline) {
-
 	  execl(
 #ifdef ENABLE_CHILLISCRIPT
 		SBINDIR "/chilli_script", SBINDIR "/chilli_script", _options.binconfig, 
@@ -225,6 +223,10 @@ int options_fromfd(int fd, bstring bt) {
   char has_error = 1;
   size_t len;
   int i;
+
+#ifdef ENABLE_MODULES
+  char isReload[MAX_MODULES];
+#endif
   
   int rd = safe_read(fd, &o, sizeof(o));
 
@@ -251,7 +253,8 @@ int options_fromfd(int fd, bstring bt) {
   close(fd);
 
   if (has_error) {
-    log_err(errno, "could not read configuration, some kind of mismatch fd=%d", fd);
+    log_err(errno, "could not read configuration, some kind of mismatch fd=%d %s",
+	    fd, SBINDIR);
     return 0;
   }
   
@@ -320,6 +323,9 @@ int options_fromfd(int fd, bstring bt) {
 #ifdef ENABLE_MODULES
   if (!option_s_l(bt, &o.moddir)) return 0;
 #endif
+#ifdef ENABLE_REDIRINJECT
+  if (!option_s_l(bt, &o.inject)) return 0;
+#endif
 
   if (!option_s_l(bt, &o.adminuser)) return 0;
   if (!option_s_l(bt, &o.adminpasswd)) return 0;
@@ -342,24 +348,33 @@ int options_fromfd(int fd, bstring bt) {
 
 #ifdef ENABLE_CHILLIREDIR
   for (i = 0; i < MAX_REGEX_PASS_THROUGHS; i++) {
+#if defined (__FreeBSD__) || defined (__APPLE__) || defined (__OpenBSD__) || defined (__NetBSD__)
+    regfree(&_options.regex_pass_throughs[i].re_host);
+    regfree(&_options.regex_pass_throughs[i].re_path);
+    regfree(&_options.regex_pass_throughs[i].re_qs);
+#else
     if (_options.regex_pass_throughs[i].re_host.allocated)
       regfree(&_options.regex_pass_throughs[i].re_host);
     if (_options.regex_pass_throughs[i].re_path.allocated)
       regfree(&_options.regex_pass_throughs[i].re_path);
     if (_options.regex_pass_throughs[i].re_qs.allocated)
       regfree(&_options.regex_pass_throughs[i].re_qs);
+#endif
   }
 #endif
 
 #ifdef ENABLE_MODULES
   for (i=0; i < MAX_MODULES; i++) {
+    isReload[i]=0;
     if (!_options.modules[i].name[0]) break;
     if (!_options.modules[i].ctx) continue;
     else {
       struct chilli_module *m = 
 	(struct chilli_module *)_options.modules[i].ctx;
+      if (!strcmp(_options.modules[i].name, o.modules[i].name))
+	isReload[i]=1;
       if (m->destroy)
-	m->destroy();
+	m->destroy(isReload[i]);
     }
     log_dbg("Unloading module %s",_options.modules[i].name);
     chilli_module_unload(_options.modules[i].ctx);
@@ -381,7 +396,7 @@ int options_fromfd(int fd, bstring bt) {
       struct chilli_module *m = 
 	(struct chilli_module *)_options.modules[i].ctx;
       if (m->initialize)
-	m->initialize(_options.modules[i].conf); 
+	m->initialize(_options.modules[i].conf, isReload[i]); 
     }
   }
 #endif
@@ -480,6 +495,9 @@ int options_save(char *file, bstring bt) {
 #ifdef ENABLE_MODULES
   if (!option_s_s(bt, &o.moddir)) return 0;
 #endif
+#ifdef ENABLE_REDIRINJECT
+  if (!option_s_s(bt, &o.inject)) return 0;
+#endif
 
   if (!option_s_s(bt, &o.adminuser)) return 0;
   if (!option_s_s(bt, &o.adminpasswd)) return 0;
@@ -598,6 +616,23 @@ void options_destroy() {
 
 void options_cleanup() {
   char file[128];
+
+#ifdef ENABLE_MODULES
+  int i;
+  for (i=0; i < MAX_MODULES; i++) {
+    if (!_options.modules[i].name[0]) break;
+    if (!_options.modules[i].ctx) continue;
+    else {
+      struct chilli_module *m = 
+	(struct chilli_module *)_options.modules[i].ctx;
+      if (m->destroy)
+	m->destroy(0);
+    }
+    log_dbg("Unloading module %s",_options.modules[i].name);
+    chilli_module_unload(_options.modules[i].ctx);
+  }
+#endif
+
   chilli_binconfig(file, sizeof(file), getpid());
   log_dbg("Removing %s", file);
   if (remove(file)) log_dbg("remove(%s) failed", file);
