@@ -1,6 +1,6 @@
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,8 @@
 
 #ifdef ENABLE_JSON
 int session_redir_json_fmt(bstring json, char *userurl, char *redirurl, 
-			   bstring logouturl, uint8_t *hismac) {
+			   bstring logouturl, uint8_t *hismac, 
+			   struct in_addr *hisip) {
   bcatcstr(json,",\"redir\":{\"originalURL\":\"");
   bcatcstr(json, userurl?userurl:"");
   bcatcstr(json,"\",\"redirectionURL\":\"");
@@ -34,6 +35,11 @@ int session_redir_json_fmt(bstring json, char *userurl, char *redirurl,
     bcatcstr(json,"\",\"logoutURL\":\"");
     bconcat(json, logouturl);
   }
+  bcatcstr(json,"\",\"ipAddress\":\"");
+  bcatcstr(json, inet_ntoa(*hisip));
+#ifdef ENABLE_LAYER3
+  if (!_options.layer3) {
+#endif
   bcatcstr(json,"\",\"macAddress\":\"");
   if (hismac) {
     char mac[REDIR_MACSTRLEN+1];
@@ -43,31 +49,25 @@ int session_redir_json_fmt(bstring json, char *userurl, char *redirurl,
 		  (unsigned int)hismac[4], (unsigned int)hismac[5]);
     bcatcstr(json, mac);
   }
+#ifdef ENABLE_LAYER3
+  }
+#endif
   bcatcstr(json,"\"}");
   return 0;
 }
 
-int session_json_fmt(struct session_state *state, 
-		     struct session_params *params,
-		     bstring json, int init) {
+int session_json_params(struct session_state *state, 
+			struct session_params *params,
+			bstring json, int init) {
   bstring tmp = bfromcstr("");
   time_t starttime = state->start_time;
-  uint32_t inoctets = state->input_octets;
-  uint32_t outoctets = state->output_octets;
-  uint32_t ingigawords = (state->input_octets >> 32);
-  uint32_t outgigawords = (state->output_octets >> 32);
-  uint32_t sessiontime;
-  uint32_t idletime;
   
-  sessiontime = mainclock_diffu(state->start_time);
-  idletime    = mainclock_diffu(state->last_sent_time);
-
-  bcatcstr(json,",\"session\":{\"sessionId\":\"");
+  bcatcstr(json,"\"sessionId\":\"");
   bcatcstr(json,state->sessionid);
   bcatcstr(json,"\",\"userName\":\"");
   bcatcstr(json,state->redir.username);
   bcatcstr(json, "\",\"startTime\":");
-  bassignformat(tmp, "%ld", init ? mainclock_now() : starttime);
+  bassignformat(tmp, "%ld", mainclock_towall(init ? mainclock_now() : starttime));
   bconcat(json, tmp);
   bcatcstr(json,",\"sessionTimeout\":");
   bassignformat(tmp, "%lld", params->sessiontimeout);
@@ -97,9 +97,28 @@ int session_json_fmt(struct session_state *state,
     bassignformat(tmp, "%lld", params->maxtotaloctets);
     bconcat(json, tmp);
   }
-  bcatcstr(json,"}");
 
-  bcatcstr(json,",\"accounting\":{\"sessionTime\":");
+  bdestroy(tmp);
+  return 0;
+}
+
+int session_json_acct(struct session_state *state, 
+		     struct session_params *params,
+		     bstring json, int init) {
+  bstring tmp = bfromcstr("");
+  uint32_t inoctets = state->input_octets;
+  uint32_t outoctets = state->output_octets;
+  uint32_t ingigawords = (state->input_octets >> 32);
+  uint32_t outgigawords = (state->output_octets >> 32);
+  uint32_t sessiontime;
+  uint32_t idletime;
+  
+  sessiontime = mainclock_diffu(state->start_time);
+  idletime    = mainclock_diffu(state->last_sent_time);
+
+  init = init || !state->authenticated;
+
+  bcatcstr(json,"\"sessionTime\":");
   bassignformat(tmp, "%ld", init ? 0 : sessiontime);
   bconcat(json, tmp);
   bcatcstr(json,",\"idleTime\":");
@@ -117,10 +136,26 @@ int session_json_fmt(struct session_state *state,
   bcatcstr(json,",\"outputGigawords\":");
   bassignformat(tmp, "%ld", init ? 0 : outgigawords);
   bconcat(json, tmp);
-  bcatcstr(json,"}");
+  bassignformat(tmp, ",\"viewPoint\":\"%s\"", 
+		_options.swapoctets ? "nas" : "client");
+  bconcat(json, tmp);
 
   bdestroy(tmp);
   return 0;
 }
+
+int session_json_fmt(struct session_state *state, 
+		     struct session_params *params,
+		     bstring json, int init) {
+  bcatcstr(json,",\"session\":{");
+  session_json_params(state,params,json,init);
+  bcatcstr(json,"}");
+
+  bcatcstr(json,",\"accounting\":{");
+  session_json_acct(state,params,json,init);
+  bcatcstr(json,"}");
+
+  return 0;
+}
 #endif
 
