@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2009-2010 Coova Technologies, LLC.
+ * Copyright (C) 2009-2011 Coova Technologies, LLC.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,20 +18,26 @@
 #include "system.h"
 #ifdef HAVE_SSL
 #include "chilli.h"
+#include "debug.h"
 
+static int openssl_init = 0;
 static openssl_env * sslenv_svr = 0;
 static openssl_env * sslenv_cli = 0;
 
 openssl_env * initssl() {
   if (sslenv_svr == 0) {
+    if (openssl_init == 0) {
+      openssl_init = 1;
 #ifdef HAVE_OPENSSL
-    SSL_library_init();
-    if (_options.debug) SSL_load_error_strings();
-    SSLeay_add_all_algorithms();
-    SSLeay_add_ssl_algorithms();
+      SSL_library_init();
+      if (_options.debug) SSL_load_error_strings();
+      SSLeay_add_all_algorithms();
+      SSLeay_add_ssl_algorithms();
 #else
-    matrixSslOpen();
+      matrixSslOpen();
+      log_dbg("MatrixSslOpen()");
 #endif
+    }
     openssl_env_init(sslenv_svr = calloc(1, sizeof(openssl_env)), 0, 1);
   }
   return sslenv_svr;
@@ -39,14 +45,18 @@ openssl_env * initssl() {
 
 openssl_env * initssl_cli() {
   if (sslenv_cli == 0) {
+    if (openssl_init == 0) {
+      openssl_init = 1;
 #ifdef HAVE_OPENSSL
-    SSL_library_init();
-    if (_options.debug) SSL_load_error_strings();
-    SSLeay_add_all_algorithms();
-    SSLeay_add_ssl_algorithms();
+      SSL_library_init();
+      if (_options.debug) SSL_load_error_strings();
+      SSLeay_add_all_algorithms();
+      SSLeay_add_ssl_algorithms();
 #else
-    matrixSslOpen();
+      matrixSslOpen();
+      log_dbg("MatrixSslOpen()");
 #endif
+    }
     openssl_env_init(sslenv_cli = calloc(1, sizeof(openssl_env)), 0, 0);
   }
   return sslenv_cli;
@@ -92,7 +102,9 @@ openssl_use_privatekey(openssl_env *env, char *file) {
   log_err(errno, "could not load private key file %s (%d,%d)\n",file,err1,err2);
   bio_err = BIO_new_fp(stderr, BIO_NOCLOSE);
   BIO_printf(bio_err,"unable to set private key file\n");
+#if(_debug_)
   ERR_print_errors(bio_err);
+#endif
   return 0;
 }
 
@@ -213,11 +225,13 @@ _openssl_env_init(openssl_env *env, char *engine, int server) {
 }
 #endif
 
+#ifdef HAVE_OPENSSL
 static int _openssl_passwd(char *buf, int size, int rwflag, void *ud) {
-  strncpy(buf, _options.sslkeypass, size);
+  safe_strncpy(buf, _options.sslkeypass, size);
   memset(_options.sslkeypass,'x',strlen(_options.sslkeypass));
   return 0;
 }
+#endif
 
 int
 openssl_env_init(openssl_env *env, char *engine, int server) {
@@ -252,10 +266,11 @@ openssl_env_init(openssl_env *env, char *engine, int server) {
     return err;
   }
 #else
-  log_dbg("cert: %s",_options.sslcertfile);
-  log_dbg("key: %s",_options.sslkeyfile);
-  log_dbg("pass: %s",_options.sslkeypass?_options.sslkeypass:"null");
-  log_dbg("ca: %s",_options.sslcafile?_options.sslcafile:"null");
+  log_dbg("MatrixSSL Setup:");
+  log_dbg("SSL cert: %s",_options.sslcertfile);
+  log_dbg("SSL key: %s",_options.sslkeyfile);
+  log_dbg("SSL pass: %s",_options.sslkeypass?_options.sslkeypass:"null");
+  log_dbg("SSL ca: %s",_options.sslcafile?_options.sslcafile:"null");
   if ( matrixSslReadKeys( &env->keys,
 			  _options.sslcertfile, 
 			  _options.sslkeyfile, 
@@ -272,6 +287,7 @@ openssl_env_init(openssl_env *env, char *engine, int server) {
 
 #ifdef HAVE_MATRIXSSL
 static int certValidator(sslCertInfo_t *t, void *arg) {
+  log_dbg("MatrixSSL: certValidator()");
   return 1;
 }
 #endif
@@ -284,7 +300,7 @@ openssl_connect_fd(openssl_env *env, int fd, int timeout) {
   c->env = env;
 #ifdef HAVE_OPENSSL
   c->con = (SSL *)SSL_new(env->ctx); 
-#else
+#elif  HAVE_MATRIXSSL
   c->con = (SSL *)SSL_new(env->keys, 0); 
 #endif
   c->sock = fd;
@@ -298,17 +314,19 @@ openssl_connect_fd(openssl_env *env, int fd, int timeout) {
 
   if (SSL_connect(c->con) < 0) {
     char is_error = 0;
+#if(_debug_)
     unsigned long error;
     while ((error = ERR_get_error())) {
       log_dbg("TLS: %s", ERR_error_string(error, NULL));
       is_error = 1;
     }
+#endif
     if (is_error) {
       openssl_free(c);
       return 0;
     }
   }
-#else
+#elif  HAVE_MATRIXSSL
   if (!SSL_connect(c->con, certValidator, c)) {
     log_err(errno, "openssl_connect_fd");
     openssl_free(c);
@@ -320,7 +338,8 @@ openssl_connect_fd(openssl_env *env, int fd, int timeout) {
 }
 
 int
-openssl_check_accept(openssl_con *c) {
+openssl_check_accept(openssl_con *c, struct redir_conn_t *conn) {
+
 #ifdef HAVE_OPENSSL  
   X509 *peer_cert;
   int rc;
@@ -360,12 +379,13 @@ openssl_check_accept(openssl_con *c) {
 	X509_NAME_oneline(X509_get_subject_name(peer_cert),subj,sizeof(subj));
 	
 	if (SSL_get_verify_result(c->con) != X509_V_OK) {
-	  log_dbg("auth_failed: %s\n", subj);
+	  log_dbg("auth_failed: %s", subj);
 	  X509_free(peer_cert);
 	  return -1;
 	}
-	
-	log_dbg("auth_success: %s\n", subj);
+		
+	log_dbg("auth_success: %s", subj);
+	if (conn) conn->s_params.flags |= ADMIN_LOGIN;
 	
 	if (_options.debug) {
 	  EVP_PKEY *pktmp = X509_get_pubkey(peer_cert);
@@ -384,17 +404,33 @@ openssl_check_accept(openssl_con *c) {
 	}
 	
 	X509_free(peer_cert);
+      } else {
+	log_dbg("no SSL certificate");
       }
     }
   }
+#elif  HAVE_MATRIXSSL
+
+  if (!c || !c->con) return -1;
+
+  if (!SSL_is_init_finished(c->con)) {
+
+    if (SSL_accept2(c->con) < 0) {
+      return -1;
+    }
+
+    if (!SSL_is_init_finished(c->con))
+      return 1;
+  }
+
 #endif
 
   return 0;
 }
 
 openssl_con *
-openssl_accept_fd(openssl_env *env, int fd, int timeout) {
-  openssl_con *c = (openssl_con *)calloc(1, sizeof(*c));
+openssl_accept_fd(openssl_env *env, int fd, int timeout, struct redir_conn_t *conn) {
+  openssl_con *c = (openssl_con *)calloc(1, sizeof(openssl_con));
   int rc;
 
   if (!c) return 0;
@@ -407,7 +443,7 @@ openssl_accept_fd(openssl_env *env, int fd, int timeout) {
   c->env = env;
 #ifdef HAVE_OPENSSL
   c->con = (SSL *)SSL_new(env->ctx); 
-#else
+#elif  HAVE_MATRIXSSL
   c->con = (SSL *)SSL_new(env->keys, SSL_FLAGS_SERVER); 
 #endif
   c->sock = fd;
@@ -423,23 +459,30 @@ openssl_accept_fd(openssl_env *env, int fd, int timeout) {
 
   SSL_set_verify_result(c->con, X509_V_OK);
 
-  if ((rc = openssl_check_accept(c)) < 0) {
+  if ((rc = openssl_check_accept(c, conn)) < 0) {
     SSL_set_shutdown(c->con, SSL_RECEIVED_SHUTDOWN);
     openssl_free(c);
     return 0;
   }
-#else
+
+#elif  HAVE_MATRIXSSL
   
   /* ndelay_off(c->sock); */
 
-  if ((rc = SSL_accept(c->con)) <= 0) {
-    log_err(errno, "SSL accept failure");
+  matrixSslSetCertValidator(c->con->ssl, certValidator, c->con->keys);
+
+  if ((rc = SSL_accept2(c->con)) < 0) {
+    log_err(c->con->status, "SSL accept failure");
     openssl_free(c);
     return 0;
   }
 
+  SSL_is_init_finished(c->con);
+
   /* ndelay_on(c->sock);*/
-  
+
+#else
+#error NO SSL SUPPORT
 #endif
 
   return c;
@@ -451,6 +494,7 @@ openssl_error(openssl_con *con, int ret, char *func) {
   int err = -1;
   if (con->con) {
     err = SSL_get_error(con->con, ret);
+#if(_debug_ > 1)
     log_dbg("SSL: (%s()) %s", func,
 	    err == SSL_ERROR_NONE ? "None": 
             err == SSL_ERROR_ZERO_RETURN ? "Return!":
@@ -460,6 +504,7 @@ openssl_error(openssl_con *con, int ret, char *func) {
 	    err == SSL_ERROR_SYSCALL ? "Syscall error, abort!":
 	    err == SSL_ERROR_SSL ? "SSL error, abort!":
 	    "Error");
+#endif
     switch (err) {
     case SSL_ERROR_NONE: return 0;
     case SSL_ERROR_WANT_READ: return 1;
@@ -573,7 +618,9 @@ openssl_write(openssl_con *con, char *b, int l, int t) {
       err = openssl_error(con, wrt, "openssl_write");
       if (err == -1) return err;
       else if (err > 0) {
+#if(_debug_)
 	log_dbg("ssl_repeart_write");
+#endif
 	goto repeat_write;
       }
       break;
@@ -600,6 +647,9 @@ openssl_free(openssl_con *con) {
 
 void
 openssl_env_free(openssl_env *env) {
+#if(_debug_)
+  log_dbg("Freeing SSL environemnt");
+#endif
 #ifdef HAVE_OPENSSL
   if (env->ctx) SSL_CTX_free(env->ctx);
   if (env->engine) ENGINE_free(env->engine);
