@@ -181,14 +181,15 @@ int loadstatus() {
 	    if (newipm) {
 	      newipm->peer = aconn;
 	      
+#ifdef ENABLE_UAMANYIP
 	      if (appconn.natip.s_addr)
 		chilli_assign_snat(aconn, 1);
+#endif
 	      
 	      dhcp_set_addrs(conn, 
 			     &newipm->addr, &_options.mask, 
 			     &aconn->ourip, &aconn->mask,
-			     &_options.dns1, &_options.dns2, 
-			     _options.domain);
+			     &_options.dns1, &_options.dns2);
 	    }
 	  }
 	  
@@ -262,14 +263,15 @@ int loadstatus() {
 	      conn->peer = aconn;
 	      newipm->peer = aconn;
 	      
+#ifdef ENABLE_UAMANYIP
 	      if (appconn.natip.s_addr)
 		chilli_assign_snat(aconn, 1);
+#endif
 	      
 	      dhcp_set_addrs(conn, 
 			     &newipm->addr, &_options.mask, 
 			     &aconn->ourip, &aconn->mask,
-			     &_options.dns1, &_options.dns2, 
-			     _options.domain);
+			     &_options.dns1, &_options.dns2);
 	    }
 	  }
 	}
